@@ -3,10 +3,13 @@
 
 # The specific tests now live in Lib/email/test
 from email.test import test_email_codecs
+from email.test import test_email_codecs_renamed
 from test import test_support
 
 def test_main():
-    test_support.run_suite(test_email_codecs.suite())
+    suite = test_email_codecs.suite()
+    suite.addTest(test_email_codecs_renamed.suite())
+    test_support.run_suite(suite)
 
 if __name__ == '__main__':
     test_main()
